function LineStrokes()
{
   /**
    * Joins path segments by extending their outside edges until
    * they meet.
    */
   var JOIN_MITER = 0;

   /**
    * Joins path segments by rounding off the corner at a radius
    * of half the line width.
    */
   var JOIN_ROUND = 1;

   /**
    * Joins path segments by connecting the outer corners of their
    * wide outlines with a straight segment.
    */
   var JOIN_BEVEL = 2;

   /**
    * Ends unclosed subpaths and dash segments with no added
    * decoration.
    */
   var CAP_BUTT = 0;

   /**
    * Ends unclosed subpaths and dash segments with a round
    * decoration that has a radius equal to half of the width
    * of the pen.
    */
   var CAP_ROUND = 1;

   /**
    * Ends unclosed subpaths and dash segments with a square
    * projection that extends beyond the end of the segment
    * to a distance equal to half of the line width.
    */
   var CAP_SQUARE = 2;
   
   /**
    * This is the list of all possible line stoke types.
    */
   var LineStrokeEnum = 
   {
      DEFAULT           : 0,
      SPECIFIED_WIDTH   : 1,
      DOTTED            : 2,
      DOTTED_SMALL      : 3,
      DOTTED_SMALL_THIN : 4
   };

   var LineStroke =
   {
      /** Dotted line specific parameter. */
      DOT_STEP            : 2.0,
      
      /** Dotted line specific parameter. */
      DOT_STEP_SMALL      : 1.0,
      
      /** Dotted line width. */
      DOT_LINE_WIDTH      : 1.0,
      
      /** Dotted thin line width. */
      DOT_LINE_THIN_WIDTH : 0.5
   };
   
   var dotStroke = new BasicStroke(LineStroke.DOT_LINE_WIDTH, CAP_BUTT, JOIN_BEVEL, 0.0,
         [LineStroke.DOT_STEP, LineStroke.DOT_STEP], LineStroke.DOT_STEP);
   var dotSmallStroke  = new BasicStroke(LineStroke.DOT_LINE_WIDTH, CAP_BUTT, JOIN_BEVEL, 0.0,
         [LineStroke.DOT_STEP_SMALL, LineStroke.DOT_STEP_SMALL], LineStroke.DOT_STEP_SMALL);
   var dotSmallThinStroke = new BasicStroke(LineStroke.DOT_LINE_THIN_WIDTH, CAP_BUTT,
         JOIN_BEVEL, 0.0, [LineStroke.DOT_STEP_SMALL, LineStroke.DOT_STEP_SMALL],
         LineStroke.DOT_STEP_SMALL);
   var defaultStroke = new BasicStroke(1.0, CAP_SQUARE, JOIN_MITER, 10.0, null, 0.0);
   
   /** javascript line */
   var caps = [ "butt", "round", "square"];
   
   /**
    *   Holds all java script styles of how two connecting segments with non-zero lengths
    *   in a shape are joined together.
    */
   var joins = [ "miter", "round", "bevel"];
   
   /**
    * Holds properties for line drawing styles.
    * 
    * @param    width
    *           The width of lines to draw.
    * @param    cap
    *           The decoration of the line ends.
    * @param    join
    *           The decoration applied where path segments meet.
    * @param    miterlimit
    *           The limit to trim the miter join.
    * @param    dash
    *           The array representing the dashing pattern.
    * @param    dash_phase
    *           The offset to start the dashing pattern.
    */
   function BasicStroke(width, cap, join, miterLimit, dash, dash_phase)
   {
      this.getWidth = function() { return width; };
      this.getCap   = function() { return caps[cap]; };
      
      this.getJoin        = function() { return joins[join]; };
      this.getMiterLimit  = function() { return miterLimit; };
      
      this.getDash       = function() { return dash; };
      this.getDashPhase  = function() { return dash_phase; };
   };

   /**
    * 
    * @param    {Number} lineStroke
    *            
    * @return   The basic line stroke object.
    */
   function getLineStroke(lineStroke)
   {
      switch(lineStroke)
      {
         case LineStrokeEnum.DOTTED:
            return dotStroke;
         case LineStrokeEnum.DOTTED_SMALL:
            return dotSmallStroke;
         case LineStrokeEnum.DOTTED_SMALL_THIN:
            return dotSmallThinStroke;
         default:
            return defaultStroke;
      }
   };
   
   /**
    * @param    context
    * @param    lineStroke
    */
   this.apply = function(context, lineStroke)
   {
      if (context instanceof CanvasRenderingContext2D)
      {
         var bs = getLineStroke(lineStroke);
         context.lineWidth = bs.getWidth();
         
         context.lineCap = bs.getCap();
         
         context.lineJoin = bs.getJoin();
         
         context.miterLimit = bs.getMiterLimit();
         if (bs.getDash())
         {
            context.setLineDash(bs.getDash());
         }
         context.lineDashOffset = bs.getDashPhase();
      }
   }

   /**
    * @param    context
    * @param    lineStroke
    * @param    path
    */
   this.applyStrokeToPath = function(context, lineStroke, path)
   {
      if (context instanceof CanvasRenderingContext2D)
      {
         PathRender(context, path, lineStroke);
      }
   }
   
   function PathRenderer(ctx, path, basicStroke)
   {
      var dash = basicStroke.getDash();
      var dashOffset = basicStroke.getDashOffset();
      var len = dash.length;
      
      var patternObj = normalize(dashOffset);
      
      var dashOn = patternObj.dashOn;
      var phase  = patternObj.startPhase;
      var idx    = patternObj.startIdx;
      var rest = dash[idx] - phase;
      for (var point in path)
      {
         if (rest < 1)
         {
            idx = (idx + 1) % len;
            rest = dash[idx];
            dashOn = !dashOn;
         }
         rest -= 1;
         var image = ctx.getImageData( point.x, point.y, 1, 1);
         var imageData = image.data;
         if (dashOn)
         {
            imageData[4] = 0xFF;
         }
         else
         {
            imageData[4] = 0;
         }
         ctx.putImageData(image, x, y);
      }
      
      /**
       * Normalizes a dash offset distance according to the given dash pattern.
       * A normalized dash offset distance is a distance that is cut the corresponding segment
       * from the dash pattern that is extended periodically in order to cover the target dash
       * offset.
       * 
       * @param    {Number} dashOffset
       *           The offset distance from the first pattern point to the point which becomes
       *           a start point of the dash pattern.
       * 
       * @return   The container {startPhase : phase, startDashOn : dashOn, startIdx : idx},
       *           where the startPhase field holds a normalized dash offset, startDashOn defines
       *           to draw or to skip, the value of startIdx is the current dash pattern index.
       */
      function normalize(dashOffset)
      {
         var phase = dashOffset;
         var idx = 0;
         var dashOn = true;
         var d;
         while (phase >= (d = dash[idx])) {
             phase -= d;
             idx = (idx + 1) % dash.length;
             dashOn = !dashOn;
         }

         return {startPhase : phase, startDashOn : dashOn, startIdx : idx};
      }
   }
}